@@ -1,4 +1,5 @@
 ''' Utility functions for algebra etc '''
+from __future__ import division, print_function, absolute_import
 
 import math
 import numpy as np
@@ -24,7 +25,6 @@ _AXES2TUPLE = {
 _TUPLE2AXES = dict((v, k) for k, v in _AXES2TUPLE.items())
 
 
-
 def sphere2cart(r, theta, phi):
     ''' Spherical to Cartesian coordinates
 
@@ -32,7 +32,7 @@ def sphere2cart(r, theta, phi):
     inclination (polar) angle, and `phi` is the azimuth angle.
 
     Imagine a sphere with center (0,0,0).  Orient it with the z axis
-    running south->north, the y axis running west-east and the x axis
+    running south-north, the y axis running west-east and the x axis
     from posterior to anterior.  `theta` (the inclination angle) is the
     angle to rotate from the z-axis (the zenith) around the y-axis,
     towards the x axis.  Thus the rotation is counter-clockwise from the
@@ -50,11 +50,11 @@ def sphere2cart(r, theta, phi):
 
     Parameters
     ------------
-    r : array-like
+    r : array_like
        radius
-    theta : array-like
+    theta : array_like
        inclination or polar angle
-    phi : array-like
+    phi : array_like
        azimuth angle
 
     Returns
@@ -94,6 +94,7 @@ def sphere2cart(r, theta, phi):
     x = r * np.cos(phi) * sin_theta
     y = r * np.sin(phi) * sin_theta
     z = r * np.cos(theta)
+    x, y, z = np.broadcast_arrays(x, y, z)
     return x, y, z
 
 
@@ -103,15 +104,15 @@ def cart2sphere(x, y, z):
     See doc for ``sphere2cart`` for angle conventions and derivation
     of the formulae.
 
-    $0 \le \theta \mathrm{(theta)} \le \pi$ and $0 \le \phi \mathrm{(phi)} \le 2 \pi$
+    $0\le\theta\mathrm{(theta)}\le\pi$ and $-\pi\le\phi\mathrm{(phi)}\le\pi$
 
     Parameters
     ------------
-    x : array-like
-       x coordinate in Cartesion space
-    y : array-like
+    x : array_like
+       x coordinate in Cartesian space
+    y : array_like
        y coordinate in Cartesian space
-    z : array-like
+    z : array_like
        z coordinate
 
     Returns
@@ -123,20 +124,33 @@ def cart2sphere(x, y, z):
     phi : array
        azimuth angle
     '''
-    r = np.sqrt(x*x + y*y + z*z)
-    theta = np.arccos(z/r)
+    r = np.sqrt(x * x + y * y + z * z)
+    theta = np.arccos(z / r)
     phi = np.arctan2(y, x)
+    r, theta, phi = np.broadcast_arrays(r, theta, phi)
     return r, theta, phi
 
 
-def normalized_vector(vec):
+def sph2latlon(theta, phi):
+    """Convert spherical coordinates to latitude and longitude.
+
+    Returns
+    -------
+    lat, lon : ndarray
+        Latitude and longitude.
+
+    """
+    return np.rad2deg(theta - np.pi / 2), np.rad2deg(phi - np.pi)
+
+
+def normalized_vector(vec, axis=-1):
     ''' Return vector divided by its Euclidean (L2) norm
 
     See :term:`unit vector` and :term:`Euclidean norm`
 
     Parameters
     ------------
-    vec : array-like shape (3,)
+    vec : array_like shape (3,)
 
     Returns
     ----------
@@ -154,51 +168,126 @@ def normalized_vector(vec):
     >>> vec.shape
     (1, 3)
     >>> normalized_vector(vec).shape
-    (3,)
+    (1, 3)
     '''
-    vec = np.asarray(vec).squeeze()
-    return vec / math.sqrt((vec**2).sum())
+    return vec / vector_norm(vec, axis, keepdims=True)
 
 
-def vector_norm(vec):
-    ''' Return vector Euclidaan (L2) norm
+def vector_norm(vec, axis=-1, keepdims=False):
+    ''' Return vector Euclidean (L2) norm
 
     See :term:`unit vector` and :term:`Euclidean norm`
 
     Parameters
     -------------
-    vec : array-like shape (3,)
+    vec : array_like
+        Vectors to norm.
+    axis : int
+        Axis over which to norm. By default norm over last axis. If `axis` is
+        None, `vec` is flattened then normed.
+    keepdims : bool
+        If True, the output will have the same number of dimensions as `vec`,
+        with shape 1 on `axis`.
 
     Returns
     ---------
-    norm : scalar
+    norm : array
+        Euclidean norms of vectors.
 
     Examples
     --------
     >>> import numpy as np
-    >>> vec = [1, 2, 3]
-    >>> l2n = np.sqrt(np.dot(vec, vec))
-    >>> nvec = vector_norm(vec)
-    >>> np.allclose(nvec, np.sqrt(np.dot(vec, vec)))
-    True
+    >>> vec = [[8, 15, 0], [0, 36, 77]]
+    >>> vector_norm(vec)
+    array([ 17.,  85.])
+    >>> vector_norm(vec, keepdims=True)
+    array([[ 17.],
+           [ 85.]])
+    >>> vector_norm(vec, axis=0)
+    array([  8.,  39.,  77.])
     '''
     vec = np.asarray(vec)
-    return math.sqrt((vec**2).sum())
-    
+    vec_norm = np.sqrt((vec * vec).sum(axis))
+    if keepdims:
+        if axis is None:
+            shape = [1] * vec.ndim
+        else:
+            shape = list(vec.shape)
+            shape[axis] = 1
+        vec_norm = vec_norm.reshape(shape)
+    return vec_norm
+
+
+def rodrigues_axis_rotation(r, theta):
+    """ Rodrigues formula
+
+    Rotation matrix for rotation around axis r for angle theta.
+
+    The rotation matrix is given by the Rodrigues formula:
+
+    R = Id + sin(theta)*Sn + (1-cos(theta))*Sn^2
+
+    with::
+
+             0  -nz  ny
+      Sn =   nz   0 -nx
+            -ny  nx   0
+
+    where n = r / ||r||
+
+    In case the angle ||r|| is very small, the above formula may lead
+    to numerical instabilities. We instead use a Taylor expansion
+    around theta=0:
+
+    R = I + sin(theta)/tetha Sr + (1-cos(theta))/teta2 Sr^2
+
+    leading to:
+
+    R = I + (1-theta2/6)*Sr + (1/2-theta2/24)*Sr^2
+
+    Parameters
+    -----------
+    r :  array_like shape (3,), axis
+    theta : float, angle in degrees
+
+    Returns
+    ----------
+    R : array, shape (3,3), rotation matrix
+
+    Examples
+    ---------
+    >>> import numpy as np
+    >>> from dipy.core.geometry import rodrigues_axis_rotation
+    >>> v=np.array([0,0,1])
+    >>> u=np.array([1,0,0])
+    >>> R=rodrigues_axis_rotation(v,40)
+    >>> ur=np.dot(R,u)
+    >>> np.round(np.rad2deg(np.arccos(np.dot(ur,u))))
+    40.0
+    """
+
+    theta = np.deg2rad(theta)
+    if theta > 1e-30:
+        n = r / np.linalg.norm(r)
+        Sn = np.array([[0, -n[2], n[1]], [n[2], 0, -n[0]], [-n[1], n[0], 0]])
+        R = np.eye(3) + np.sin(theta) * Sn + \
+            (1 - np.cos(theta)) * np.dot(Sn, Sn)
+    else:
+        Sr = np.array([[0, -r[2], r[1]], [r[2], 0, -r[0]], [-r[1], r[0], 0]])
+        theta2 = theta * theta
+        R = np.eye(3) + (1 - theta2 / 6.) * \
+            Sr + (.5 - theta2 / 24.) * np.dot(Sr, Sr)
+    return R
+
 
 def nearest_pos_semi_def(B):
     ''' Least squares positive semi-definite tensor estimation
-    
-    Reference: Niethammer M, San Jose Estepar R, Bouix S, Shenton M,
-    Westin CF.  On diffusion tensor estimation. Conf Proc IEEE Eng Med
-    Biol Soc.  2006;1:2622-5. PubMed PMID: 17946125; PubMed Central
-    PMCID: PMC2791793.
- 
+
     Parameters
     ------------
-    B : (3,3) array-like
+    B : (3,3) array_like
        B matrix - symmetric. We do not check the symmetry.
-       
+
     Returns
     ---------
     npds : (3,3) array
@@ -211,6 +300,14 @@ def nearest_pos_semi_def(B):
     array([[ 0.75,  0.  ,  0.  ],
            [ 0.  ,  0.75,  0.  ],
            [ 0.  ,  0.  ,  0.  ]])
+
+    References
+    ----------
+    .. [1] Niethammer M, San Jose Estepar R, Bouix S, Shenton M, Westin CF.
+           On diffusion tensor estimation. Conf Proc IEEE Eng Med Biol Soc.
+           2006;1:2622-5. PubMed PMID: 17946125; PubMed Central PMCID:
+           PMC2791793.
+
     '''
     B = np.asarray(B)
     vals, vecs = npl.eigh(B)
@@ -221,23 +318,23 @@ def nearest_pos_semi_def(B):
     if cardneg == 0:
         return B
     if cardneg == 3:
-        return np.zeros((3,3))
+        return np.zeros((3, 3))
     lam1a, lam2a, lam3a = vals
     scalers = np.zeros((3,))
     if cardneg == 2:
-        b112 = np.max([0,lam1a+(lam2a+lam3a)/3.])
+        b112 = np.max([0, lam1a + (lam2a + lam3a) / 3.])
         scalers[0] = b112
     elif cardneg == 1:
-        lam1b=lam1a+0.25*lam3a
-        lam2b=lam2a+0.25*lam3a
+        lam1b = lam1a + 0.25 * lam3a
+        lam2b = lam2a + 0.25 * lam3a
         if lam1b >= 0 and lam2b >= 0:
             scalers[:2] = lam1b, lam2b
-        else: # one of the lam1b, lam2b is < 0
+        else:  # one of the lam1b, lam2b is < 0
             if lam2b < 0:
-                b111=np.max([0,lam1a+(lam2a+lam3a)/3.])
+                b111 = np.max([0, lam1a + (lam2a + lam3a) / 3.])
                 scalers[0] = b111
             if lam1b < 0:
-                b221=np.max([0,lam2a+(lam1a+lam3a)/3.])
+                b221 = np.max([0, lam2a + (lam1a + lam3a) / 3.])
                 scalers[1] = b221
     # resort the scalers to match the original vecs
     scalers = scalers[np.argsort(inds)]
@@ -249,10 +346,10 @@ def sphere_distance(pts1, pts2, radius=None, check_radius=True):
 
     Parameters
     ------------
-    pts1 : (N,R) or (R,) array-like
+    pts1 : (N,R) or (R,) array_like
        where N is the number of points and R is the number of
        coordinates defining a point (``R==3`` for 3D)
-    pts2 : (N,R) or (R,) array-like
+    pts2 : (N,R) or (R,) array_like
        where N is the number of points and R is the number of
        coordinates defining a point (``R==3`` for 3D).  It should be
        possible to broadcast `pts1` against `pts2`
@@ -263,7 +360,7 @@ def sphere_distance(pts1, pts2, radius=None, check_radius=True):
        If True, check if the points are on the sphere surface - i.e
        check if the vector lengths in `pts1` and `pts2` are close to
        `radius`.  Default is True.
-       
+
     Returns
     ---------
     d : (N,) or (0,) array
@@ -274,18 +371,18 @@ def sphere_distance(pts1, pts2, radius=None, check_radius=True):
     ----------
     cart_distance : cartesian distance between points
     vector_cosine : cosine of angle between vectors
-    
+
     Examples
     ----------
-    >>> print '%.4f' % sphere_distance([0,1],[1,0])
+    >>> print('%.4f' % sphere_distance([0,1],[1,0]))
     1.5708
-    >>> print '%.4f' % sphere_distance([0,3],[3,0])
+    >>> print('%.4f' % sphere_distance([0,3],[3,0]))
     4.7124
     """
     pts1 = np.asarray(pts1)
     pts2 = np.asarray(pts2)
-    lens1 = np.sqrt(np.sum(pts1**2, axis=-1))
-    lens2 = np.sqrt(np.sum(pts2**2, axis=-1))
+    lens1 = np.sqrt(np.sum(pts1 ** 2, axis=-1))
+    lens2 = np.sqrt(np.sum(pts2 ** 2, axis=-1))
     if radius is None:
         radius = (np.mean(lens1) + np.mean(lens2)) / 2.0
     if check_radius:
@@ -305,26 +402,26 @@ def cart_distance(pts1, pts2):
     If either of `pts1` or `pts2` is 2D, then we take the first
     dimension to index points, and the second indexes coordinate.  More
     generally, we take the last dimension to be the coordinate
-    dimension. 
-    
+    dimension.
+
     Parameters
-    ------------
-    pts1 : (N,R) or (R,) array-like
+    ----------
+    pts1 : (N,R) or (R,) array_like
        where N is the number of points and R is the number of
        coordinates defining a point (``R==3`` for 3D)
-    pts2 : (N,R) or (R,) array-like
+    pts2 : (N,R) or (R,) array_like
        where N is the number of points and R is the number of
        coordinates defining a point (``R==3`` for 3D).  It should be
        possible to broadcast `pts1` against `pts2`
 
     Returns
-    ---------
+    -------
     d : (N,) or (0,) array
        Cartesian distances between corresponding points in `pts1` and
        `pts2`
 
     See also
-    ----------
+    --------
     sphere_distance : distance between points on sphere surface
 
     Examples
@@ -332,7 +429,7 @@ def cart_distance(pts1, pts2):
     >>> cart_distance([0,0,0], [0,0,3])
     3.0
     '''
-    sqs = np.subtract(pts1, pts2)**2
+    sqs = np.subtract(pts1, pts2) ** 2
     return np.sqrt(np.sum(sqs, axis=-1))
 
 
@@ -348,12 +445,12 @@ def vector_cosine(vecs1, vecs2):
 
     Parameters
     -------------
-    vecs1 : (N, R) or (R,) array-like
+    vecs1 : (N, R) or (R,) array_like
        N vectors (as rows) or single vector.  Vectors have R elements.
-    vecs1 : (N, R) or (R,) array-like
+    vecs1 : (N, R) or (R,) array_like
        N vectors (as rows) or single vector.  Vectors have R elements.
        It should be possible to broadcast `vecs1` against `vecs2`
-    
+
     Returns
     ----------
     vcos : (N,) or (0,) array
@@ -366,66 +463,69 @@ def vector_cosine(vecs1, vecs2):
     """
     vecs1 = np.asarray(vecs1)
     vecs2 = np.asarray(vecs2)
-    lens1 = np.sqrt(np.sum(vecs1**2, axis=-1))
-    lens2 = np.sqrt(np.sum(vecs2**2, axis=-1))
+    lens1 = np.sqrt(np.sum(vecs1 ** 2, axis=-1))
+    lens2 = np.sqrt(np.sum(vecs2 ** 2, axis=-1))
     dots = np.inner(vecs1, vecs2)
     lens = lens1 * lens2
     return dots / lens
 
+
 def lambert_equal_area_projection_polar(theta, phi):
-    r''' Lambert Equal Area Projection from polar sphere to plane
+    r""" Lambert Equal Area Projection from polar sphere to plane
 
     Return positions in (y1,y2) plane corresponding to the points
     with polar coordinates (theta, phi) on the unit sphere, under the
     Lambert Equal Area Projection mapping (see Mardia and Jupp (2000),
     Directional Statistics, p. 161).
-    
+
     See doc for ``sphere2cart`` for angle conventions
 
     - $0 \le \theta \le \pi$ and $0 \le \phi \le 2 \pi$
     - $|(y_1,y_2)| \le 2$
-    
-    The Lambert EAP maps the upper hemisphere to the planar disc of radius 1 
+
+    The Lambert EAP maps the upper hemisphere to the planar disc of radius 1
     and the lower hemisphere to the planar annulus between radii 1 and 2,
-    and *vice versa*.  
+    and *vice versa*.
+
     Parameters
     ----------
-    theta : array-like
+    theta : array_like
        theta spherical coordinates
-    phi : array-like
+    phi : array_like
        phi spherical coordinates
 
     Returns
     ---------
     y : (N,2) array
        planar coordinates of points following mapping by Lambert's EAP.
-    '''
+    """
 
-    return 2 * np.repeat(np.sin(theta/2),2).reshape((theta.shape[0],2)) * np.column_stack((np.cos(phi), np.sin(phi)))
+    return 2 * np.repeat(np.sin(theta / 2), 2).reshape((theta.shape[0], 2)) * np.column_stack((np.cos(phi), np.sin(phi)))
 
 
-def lambert_equal_area_projection_cart(x,y,z):
+def lambert_equal_area_projection_cart(x, y, z):
     r''' Lambert Equal Area Projection from cartesian vector to plane
 
     Return positions in $(y_1,y_2)$ plane corresponding to the
     directions of the vectors with cartesian coordinates xyz under the
     Lambert Equal Area Projection mapping (see Mardia and Jupp (2000),
     Directional Statistics, p. 161).
-    
-    The Lambert EAP maps the upper hemisphere to the planar disc of radius 1 
-    and the lower hemisphere to the planar annulus between radii 1 and 2, 
-    The Lambert EAP maps the upper hemisphere to the planar disc of radius 1 
+
+    The Lambert EAP maps the upper hemisphere to the planar disc of radius 1
+    and the lower hemisphere to the planar annulus between radii 1 and 2,
+    The Lambert EAP maps the upper hemisphere to the planar disc of radius 1
     and the lower hemisphere to the planar annulus between radii 1 and 2.
     and *vice versa*.
+
     See doc for ``sphere2cart`` for angle conventions
 
     Parameters
     ------------
-    x : array-like
+    x : array_like
        x coordinate in Cartesion space
-    y : array-like
+    y : array_like
        y coordinate in Cartesian space
-    z : array-like
+    z : array_like
        z coordinate
 
     Returns
@@ -434,12 +534,10 @@ def lambert_equal_area_projection_cart(x,y,z):
        planar coordinates of points following mapping by Lambert's EAP.
     '''
 
-    (r, theta, phi) = cart2sphere(x,y,z)
+    (r, theta, phi) = cart2sphere(x, y, z)
     return lambert_equal_area_projection_polar(theta, phi)
 
 
-
-
 def euler_matrix(ai, aj, ak, axes='sxyz'):
     """Return homogeneous rotation matrix from Euler angles and axis sequence.
 
@@ -447,13 +545,13 @@ def euler_matrix(ai, aj, ak, axes='sxyz'):
     http://www.lfd.uci.edu/~gohlke/code/transformations.py.html
 
     Parameters
-    ------------    
+    ------------
     ai, aj, ak : Euler's roll, pitch and yaw angles
     axes : One of 24 axis sequences as string or encoded tuple
 
     Returns
     ---------
-    matrix: 4x4 numpy array
+    matrix : ndarray (4, 4)
 
     Code modified from the work of Christoph Gohlke link provided here
     http://www.lfd.uci.edu/~gohlke/code/transformations.py.html
@@ -481,8 +579,8 @@ def euler_matrix(ai, aj, ak, axes='sxyz'):
         firstaxis, parity, repetition, frame = axes
 
     i = firstaxis
-    j = _NEXT_AXIS[i+parity]
-    k = _NEXT_AXIS[i-parity+1]
+    j = _NEXT_AXIS[i + parity]
+    k = _NEXT_AXIS[i - parity + 1]
 
     if frame:
         ai, ak = ak, ai
@@ -491,54 +589,59 @@ def euler_matrix(ai, aj, ak, axes='sxyz'):
 
     si, sj, sk = math.sin(ai), math.sin(aj), math.sin(ak)
     ci, cj, ck = math.cos(ai), math.cos(aj), math.cos(ak)
-    cc, cs = ci*ck, ci*sk
-    sc, ss = si*ck, si*sk
+    cc, cs = ci * ck, ci * sk
+    sc, ss = si * ck, si * sk
 
     M = np.identity(4)
     if repetition:
         M[i, i] = cj
-        M[i, j] = sj*si
-        M[i, k] = sj*ci
-        M[j, i] = sj*sk
-        M[j, j] = -cj*ss+cc
-        M[j, k] = -cj*cs-sc
-        M[k, i] = -sj*ck
-        M[k, j] = cj*sc+cs
-        M[k, k] = cj*cc-ss
+        M[i, j] = sj * si
+        M[i, k] = sj * ci
+        M[j, i] = sj * sk
+        M[j, j] = -cj * ss + cc
+        M[j, k] = -cj * cs - sc
+        M[k, i] = -sj * ck
+        M[k, j] = cj * sc + cs
+        M[k, k] = cj * cc - ss
     else:
-        M[i, i] = cj*ck
-        M[i, j] = sj*sc-cs
-        M[i, k] = sj*cc+ss
-        M[j, i] = cj*sk
-        M[j, j] = sj*ss+cc
-        M[j, k] = sj*cs-sc
+        M[i, i] = cj * ck
+        M[i, j] = sj * sc - cs
+        M[i, k] = sj * cc + ss
+        M[j, i] = cj * sk
+        M[j, j] = sj * ss + cc
+        M[j, k] = sj * cs - sc
         M[k, i] = -sj
-        M[k, j] = cj*si
-        M[k, k] = cj*ci
+        M[k, j] = cj * si
+        M[k, k] = cj * ci
     return M
 
 
-def compose_matrix(scale=None, shear=None, angles=None, translate=None,perspective=None):
+def compose_matrix(scale=None, shear=None, angles=None, translate=None, perspective=None):
     """Return 4x4 transformation matrix from sequence of
     transformations.
 
     Code modified from the work of Christoph Gohlke link provided here
     http://www.lfd.uci.edu/~gohlke/code/transformations.py.html
 
-    This is the inverse of the decompose_matrix function.
+    This is the inverse of the ``decompose_matrix`` function.
 
     Parameters
-    -------------    
-    scale : vector of 3 scaling factors
-    shear : list of shear factors for x-y, x-z, y-z axes
-    angles : list of Euler angles about static x, y, z axes
-    translate : translation vector along x, y, z axes
-    perspective : perspective partition of matrix
+    -------------
+    scale : (3,) array_like
+        Scaling factors.
+    shear : array_like
+        Shear factors for x-y, x-z, y-z axes.
+    angles : array_like
+        Euler angles about static x, y, z axes.
+    translate : array_like
+        Translation vector along x, y, z axes.
+    perspective : array_like
+        Perspective partition of matrix.
 
     Returns
     ---------
     matrix : 4x4 array
-    
+
 
     Examples
     ----------
@@ -580,14 +683,12 @@ def compose_matrix(scale=None, shear=None, angles=None, translate=None,perspecti
     return M
 
 
-
-
 def decompose_matrix(matrix):
     """Return sequence of transformations from transformation matrix.
 
     Code modified from the excellent work of Christoph Gohlke link provided here
     http://www.lfd.uci.edu/~gohlke/code/transformations.py.html
-    
+
     Parameters
     ------------
     matrix : array_like
@@ -595,15 +696,21 @@ def decompose_matrix(matrix):
 
     Returns
     ---------
-    tuple of:    
-        scale : vector of 3 scaling factors
-        shear : list of shear factors for x-y, x-z, y-z axes
-        angles : list of Euler angles about static x, y, z axes
-        translate : translation vector along x, y, z axes
-        perspective : perspective partition of matrix
+    scale : (3,) ndarray
+        Three scaling factors.
+    shear : (3,) ndarray
+        Shear factors for x-y, x-z, y-z axes.
+    angles : (3,) ndarray
+        Euler angles about static x, y, z axes.
+    translate : (3,) ndarray
+        Translation vector along x, y, z axes.
+    perspective : ndarray
+        Perspective partition of matrix.
 
-    
-    Raise ValueError if matrix is of wrong type or degenerative.
+    Raises
+    ------
+    ValueError
+        If matrix is of wrong type or degenerative.
 
     Examples
     -----------
@@ -666,50 +773,7 @@ def decompose_matrix(matrix):
 
     return scale, shear, angles, translate, perspective
 
-def vector_norm(data, axis=None, out=None):
-    """Return length, i.e. euclidean norm, of ndarray along axis.
 
-    Examples
-    ----------
-    
-    >>> import numpy
-    >>> v = numpy.random.random(3)
-    >>> n = vector_norm(v)
-    >>> numpy.allclose(n, numpy.linalg.norm(v))
-    True
-    >>> v = numpy.random.rand(6, 5, 3)
-    >>> n = vector_norm(v, axis=-1)
-    >>> numpy.allclose(n, numpy.sqrt(numpy.sum(v*v, axis=2)))
-    True
-    >>> n = vector_norm(v, axis=1)
-    >>> numpy.allclose(n, numpy.sqrt(numpy.sum(v*v, axis=1)))
-    True
-    >>> v = numpy.random.rand(5, 4, 3)
-    >>> n = numpy.empty((5, 3), dtype=numpy.float64)
-    >>> vector_norm(v, axis=1, out=n)
-    >>> numpy.allclose(n, numpy.sqrt(numpy.sum(v*v, axis=1)))
-    True
-    >>> vector_norm([])
-    0.0
-    >>> vector_norm([1.0])
-    1.0
-
-    """
-    data = np.array(data, dtype=np.float64, copy=True)
-    if out is None:
-        if data.ndim == 1:
-            return math.sqrt(np.dot(data, data))
-        data *= data
-        out = np.atleast_1d(np.sum(data, axis=axis))
-        np.sqrt(out, out)
-        return out
-    else:
-        data *= data
-        np.sum(data, axis=axis, out=out)
-        np.sqrt(out, out)
-
-
-        
 def circumradius(a, b, c):
     ''' a, b and c are 3-dimensional vectors which are the vertices of a
     triangle. The function returns the circumradius of the triangle, i.e
@@ -719,23 +783,84 @@ def circumradius(a, b, c):
 
     Parameters
     ----------
-    a, b, c : (3,) arraylike
+    a, b, c : (3,) array_like
        the three vertices of the triangle
-    
+
     Returns
     -------
     circumradius : float
         the desired circumradius
     '''
-    x = a-c
-    xx = np.linalg.norm(x)**2
-    y = b-c
-    yy = np.linalg.norm(y)**2
-    z = np.cross(x,y)
+    x = a - c
+    xx = np.linalg.norm(x) ** 2
+    y = b - c
+    yy = np.linalg.norm(y) ** 2
+    z = np.cross(x, y)
     # test for collinearity
     if np.linalg.norm(z) == 0:
-        return np.sqrt(np.max(np.dot(x,x), np.dot(y,y), np.dot(a-b,a-b)))/2.
+        return np.sqrt(np.max(np.dot(x, x), np.dot(y, y), np.dot(a - b, a - b))) / 2.
     else:
-        m = np.vstack((x,y,z))
-        w = np.dot(np.linalg.inv(m.T),np.array([xx/2.,yy/2.,0]))
-        return np.linalg.norm(w)/2.
+        m = np.vstack((x, y, z))
+        w = np.dot(np.linalg.inv(m.T), np.array([xx / 2., yy / 2., 0]))
+        return np.linalg.norm(w) / 2.
+
+
+def vec2vec_rotmat(u, v):
+    r""" rotation matrix from 2 unit vectors
+
+    u,v being unit 3d vectors return a 3x3 rotation matrix R than aligns u to v.
+
+    In general there are many rotations that will map u to v. If S is any
+    rotation using v as an axis then R.S will also map u to v since (S.R)u =
+    S(Ru) = Sv = v.  The rotation R returned by vec2vec_rotmat leaves fixed the
+    perpendicular to the plane spanned by u and v.
+
+    The transpose of R will align v to u.
+
+    Parameters
+    -----------
+    u : array, shape(3,)
+    v : array, shape(3,)
+
+    Returns
+    ---------
+    R : array, shape(3,3)
+
+    Examples
+    ---------
+    >>> import numpy as np
+    >>> from dipy.core.geometry import vec2vec_rotmat
+    >>> u=np.array([1,0,0])
+    >>> v=np.array([0,1,0])
+    >>> R=vec2vec_rotmat(u,v)
+    >>> np.dot(R,u)
+    array([ 0.,  1.,  0.])
+    >>> np.dot(R.T,v)
+    array([ 1.,  0.,  0.])
+
+    """
+
+    # return eye when u is the same with v
+    if np.linalg.norm(u - v) < np.finfo(float).eps:
+        return np.eye(3)
+
+    w = np.cross(u, v)
+    w = w / np.linalg.norm(w)
+
+    # vp is in plane of u,v,  perpendicular to u
+    vp = (v - (np.dot(u, v) * u))
+    vp = vp / np.linalg.norm(vp)
+
+    # (u vp w) is an orthonormal basis
+    P = np.array([u, vp, w])
+    Pt = P.T
+    cosa = np.dot(u, v)
+    sina = np.sqrt(1 - cosa ** 2)
+    R = np.array([[cosa, -sina, 0], [sina, cosa, 0], [0, 0, 1]])
+    Rp = np.dot(Pt, np.dot(R, P))
+
+    # make sure that you don't return any Nans
+    if np.sum(np.isnan(Rp)) > 0:
+        return np.eye(3)
+
+    return Rp
