@@ -1,4 +1,6 @@
-import cPickle
+""" Load and save pickles """
+# Python 3 merged cPickle into pickle
+from ..utils.six.moves import cPickle
 
 def save_pickle(fname,dix):
     ''' Save `dix` to `fname` as pickle
@@ -30,7 +32,7 @@ def save_pickle(fname,dix):
     
     '''
     out=open(fname,'wb')
-    cPickle.dump(dix,out)
+    cPickle.dump(dix,out,protocol=cPickle.HIGHEST_PROTOCOL)
     out.close()
 
 
