@@ -1,3 +1,5 @@
+from __future__ import division, print_function, absolute_import
+
 import numpy as np
 import nose
 from nose.tools import assert_true, assert_false, assert_equal, assert_almost_equal
@@ -36,14 +38,14 @@ def test_LSCv2():
     t1=time()
     C3=pf.local_skeleton_clustering(T,.5)
     t2=time()
-    print t2-t1
-    print len(C3)
+    print(t2-t1)
+    print(len(C3))
     
     t1=time()
     C4=pf.local_skeleton_clustering_3pts(T,.5)
     t2=time()
-    print t2-t1
-    print len(C4)
+    print(t2-t1)
+    print(len(C4))
 
     for c in C3:
         assert_equal(np.sum(C3[c]['hidden']-C4[c]['hidden']),0)
@@ -55,14 +57,14 @@ def test_LSCv2():
     t1=time()
     C5=pf.local_skeleton_clustering(T2,.5)
     t2=time()
-    print t2-t1
-    print len(C5)
+    print(t2-t1)
+    print(len(C5))
     
     from dipy.data import get_data
     from nibabel import trackvis as tv
     try:
         from dipy.viz import fvtk
-    except ImportError, e:
+    except ImportError as e:
         raise nose.plugins.skip.SkipTest(
             'Fails to import dipy.viz due to %s' % str(e))
     
@@ -70,11 +72,11 @@ def test_LSCv2():
     T3=[tm.downsample(s[0],6) for s in streams]    
     
 
-    print 'lenT3',len(T3)
+    print('lenT3',len(T3))
     
     C=pf.local_skeleton_clustering(T3,10.)
     
-    print 'lenC',len(C)
+    print('lenC',len(C))
     
     """
     
@@ -116,7 +118,45 @@ def test_bundles_distances_mam():
     for metric in ('avg', 'min', 'max'):       
         DM2 = pf.bundles_distances_mam(tracksA, tracksB, metric=metric)
         
+def test_bundles_distances_mdf():
+    xyz1A = np.array([[0,0,0],[1,0,0],[2,0,0]],dtype='float32')    
+    xyz2A = np.array([[0,1,1],[1,0,1],[2,3,-2]],dtype='float32')
+    xyz3A = np.array([[0,0,0],[1,0,0],[3,0,0]],dtype='float32')    
+    xyz1B = np.array([[-1,0,0],[2,0,0],[2,3,0]],dtype='float32')
+       
+    tracksA = [xyz1A,xyz2A]
+    tracksB = [xyz1B, xyz1A, xyz2A]           
+    DM2 = pf.bundles_distances_mdf(tracksA, tracksB)
     
+    tracksA = [xyz1A,xyz1A]
+    tracksB = [xyz1A,xyz1A]
+    
+    DM2 = pf.bundles_distances_mdf(tracksA, tracksB)
+    assert_array_almost_equal(DM2,np.zeros((2,2)))
+    
+    tracksA = [xyz1A,xyz3A]
+    tracksB = [xyz2A]
+    
+    DM2 = pf.bundles_distances_mdf(tracksA, tracksB)
+    print(DM2)
+        
+    #assert_array_almost_equal(DM2,np.zeros((2,2)))    
+    DM=np.zeros(DM2.shape)
+    for (a,ta) in enumerate(tracksA):
+        for (b,tb) in enumerate(tracksB):
+            md=np.sum(np.sqrt(np.sum((ta-tb)**2,axis=1)))/3.
+            md2=np.sum(np.sqrt(np.sum((ta-tb[::-1])**2,axis=1)))/3.
+            DM[a,b]=np.min((md,md2))
+    print(DM)
+    
+    print('--------------')
+    for t in tracksA:
+        print(t)
+    print('--------------')
+    for t in tracksB:
+        print(t)
+        
+    assert_array_almost_equal(DM,DM2,4)
     
 def test_mam_distances():
     xyz1 = np.array([[0,0,0],[1,0,0],[2,0,0],[3,0,0]])
